/*
 * @(#)BinFileResource.java
 *
 * Copyright (c) 2005 masahito suzuki, Inc. All Rights Reserved
 */
package com.JRcServer.commons.resource ;

import com.JRcServer.commons.def.BaseDef;
import com.JRcServer.commons.exception.InputException;
import com.JRcServer.commons.io.FileAccessException;
import com.JRcServer.commons.io.IOCom;
import com.JRcServer.commons.io.RandomIO;
import com.JRcServer.commons.thread.Synchronized;
import com.JRcServer.commons.util.UtilCom;

/**
 * ファイル領域を利用したバイナリオブジェクト.
 * <BR><BR>
 * ファイル領域を利用したバイナリオブジェクトを生成します.
 *
 * @version     1.00, 2005/04/01
 * @author      Masahito Suzuki
 * @since  JRcCommons 1.00
 */
public class BinFileResource implements BinResource
{
    /**
     * バイナリ単位.
     */
    private static final int BINARY_LENGTH = 512 ;
    
        
    /**
     * ファイルポインター.
     */
    private final RandomIO m_fp = new RandomIO() ;
    
    /**
     * 全体データ長.
     */
    private int m_allLength = -1 ;
     
    
    /**
     * 有効書き込みデータ位置.
     */
    private int m_usePoint = 0 ;
    
    /**
     * ランダムファイル名.
     */
    private String m_randName = null ;
    
    /**
     * ファイル削除フラグ.
     */
    private boolean m_delFlg = false ;
    
    /**
     * 同期オブジェクト.
     */
    private final Synchronized m_sync = new Synchronized() ;
    
    
    
    /**
     * コンストラクタ.
     */
    public BinFileResource(){}
    
    /**
     * コンストラクタ.
     * <BR><BR>
     * ディレクトリ名を設定して、バイナリオブジェクトを生成します.<BR>
     * また、生成されるファイル名はランダムで生成されます.
     * <BR>
     * @param dir 対象ディレクトリ名を設定します.
     * @param size 生成ファイルサイズを設定します.
     * @exception InputException 入力例外.
     */
    public BinFileResource( String dir,int size )
        throws InputException
    {
        this.create( dir,size ) ;
    }
    
    /**
     * コンストラクタ.
     * <BR><BR>
     * 対象ファイル名を設定してバイナリリソースを生成します.
     * <BR>
     * @param name 対象のファイル名を設定します.
     * @exception InputException 入力例外.
     */
    public BinFileResource( String name )
        throws InputException
    {
        this.create( name ) ;
    }
    
    /**
     * コンストラクタ.
     * <BR><BR>
     * 対象ファイル名を設定してバイナリリソースを生成します.
     * <BR>
     * @param mode 生成モードを設定します.<BR>
     *             [true]を設定した場合、対象ファイル名を示すファイルが
     *             存在しない場合、新規に作成します.<BR>
     *             [false]を設定した場合、対象ファイル名を示すファイルが
     *             存在しない場合、エラーとします.
     * @param name 対象のファイル名を設定します.
     * @exception InputException 入力例外.
     */
    public BinFileResource( boolean mode,String name )
        throws InputException
    {
        this.create( mode,name ) ;
    }
    
    /**
     * ファイナライズ処理定義.
     * <BR><BR>
     * ファイナライズ処理定義.
     * <BR>
     * @exception Exception 例外処理が返されます.
     */
    protected final void finalize() throws Exception
    {
        
        try{
            this.clear() ;
        }catch( Exception t ){
        }
        
    }
    
    /**
     * 情報生成.
     * <BR><BR>
     * ディレクトリ名を設定して、バイナリオブジェクトを生成します.<BR>
     * また、生成されるファイル名はランダムで生成されます.
     * <BR>
     * @param dir 対象ディレクトリ名を設定します.
     * @param size 生成ファイルサイズを設定します.
     * @exception InputException 入力例外.
     */
    public final void create( String dir,int size )
        throws InputException
    {
        if( dir == null || size <= 0 ){
            throw new InputException( "引数は不正です" ) ;
        }
        
        this.clear() ;
        m_sync.create() ;
        
        try{
            
            synchronized( m_sync.get() ){
                
                if( IOCom.isDirExists( dir ) == false ){
                    throw new InputException(
                        "ディレクトリ名(" + dir +
                        ")は存在しません"
                    ) ;
                }
                
                dir = IOCom.getFullPath( dir ) ;
                m_randName = BinFileResource.createRandomFile( dir,m_fp ) ;
                m_fp.setLength( size ) ;
                m_allLength = size ;
                m_delFlg = true ;
                
            }
            
        }catch( InputException in ){
            this.clear() ;
            throw in ;
        }catch( Exception e ){
            this.clear() ;
            throw new InputException( e ) ;
        }
        
    }
    
    /**
     * 情報生成.
     * <BR><BR>
     * 対象ファイル名を設定してバイナリリソースを生成します.
     * <BR>
     * @param name 対象のファイル名を設定します.
     * @exception InputException 入力例外.
     */
    public final void create( String name )
        throws InputException
    {
        this.create( false,name ) ;
    }
    
    /**
     * 情報生成.
     * <BR><BR>
     * 対象ファイル名を設定してバイナリリソースを生成します.
     * <BR>
     * @param mode 生成モードを設定します.<BR>
     *             [true]を設定した場合、対象ファイル名を示すファイルが
     *             存在しない場合、新規に作成します.<BR>
     *             [false]を設定した場合、対象ファイル名を示すファイルが
     *             存在しない場合、エラーとします.
     * @param name 対象のファイル名を設定します.
     * @exception InputException 入力例外.
     */
    public final void create( boolean mode,String name )
        throws InputException
    {
        if( name == null ){
            throw new InputException( "引数は不正です" ) ;
        }
        
        this.clear() ;
        m_sync.create() ;
        
        try{
            
            synchronized( m_sync.get() ){
                
                if( mode == false && IOCom.isFileExists( name ) == false ){
                    throw new InputException(
                        "ファイル名(" + name +
                        ")は存在しません"
                    ) ;
                }
                
                m_fp.open( name,false ) ;
                m_randName = m_fp.getFileName() ;
                m_allLength = m_usePoint = ( int )m_fp.getLength() ;
                m_delFlg = false ;
                
            }
            
        }catch( InputException in ){
            this.clear() ;
            throw in ;
        }catch( Exception e ){
            this.clear() ;
            throw new InputException( e ) ;
        }
        
    }
    
    /**
     * 情報クリア.
     * <BR><BR>
     * バイナリ情報をクリアします.
     */
    public final void clear()
    {
        String name = null ;
        
        name = m_fp.getFileName() ;
        
        m_sync.clear() ;
        
        if( m_delFlg == true && name != null ){
            
            m_fp.close() ;
            
            try{
                IOCom.deleteFile( name ) ;
            }catch( Exception e ){
            }
            
        }
        else if( m_delFlg == false && m_fp.isOpen() == true ){
            
            try{
                m_fp.setLength( ( long )m_usePoint ) ;
            }catch( Exception e ){
            }
            
            m_fp.close() ;
            
        }
        
        m_fp.close() ;
        
        m_randName = null ;
        m_allLength = -1 ;
        m_usePoint = 0 ;
        m_delFlg = false ;
        
    }
    
    /**
     * リセット処理.
     * <BR><BR>
     * 有効データ長をリセットします.
     */
    public final void reset()
    {
        try{
            synchronized( m_sync.get() ){
                m_usePoint = 0 ;
            }
        }catch( Exception e ){
        }
    }
    
    /**
     * 情報設定.
     * <BR><BR>
     * 対象条件に情報を設定します.
     * <BR>
     * @param no 設定対象項番を設定します.
     * @param b 設定対象のバイト情報を設定します.
     */
    public final void set( int no,int b )
    {
        try{
            synchronized( m_sync.get() ){
                
                this.newSize( no+1 ) ;
                m_fp.write( new byte[]{ ( byte )( b & 0x000000ff ) },no ) ;
                m_usePoint = ( m_usePoint <= no+1 ) ? no+1 : m_usePoint ;
                
            }
        }catch( ArrayIndexOutOfBoundsException ai ){
            throw ai ;
        }catch( Exception in ){
        }
    }
    
    /**
     * バイナリ情報設定.
     * <BR><BR>
     * 対象のバイナリ情報を設定します.
     * <BR>
     * @param no 設定開始位置となる項番を設定します.
     * @param bin 設定対象のバイナリ情報を設定します.
     * @return int 設定されたバイナリ長が返されます.
     * @exception ArrayIndexOutOfBoundsException 不正インデックス例外.
     */
    public final int setBinary( int no,byte[] bin ) throws ArrayIndexOutOfBoundsException
    {
        if( bin == null || no < 0 ){
            if( bin == null ){
                throw new ArrayIndexOutOfBoundsException( "設定バイナリデータは不正です" ) ;
            }
            throw new ArrayIndexOutOfBoundsException(
                "指定位置(" + no + "は範囲外です"
            ) ;
        }
        else if( bin.length <= 0 ){
            return 0 ;
        }
        
        return this.setBinary( no,bin,0,bin.length ) ;
    }
    
    /**
     * バイナリ情報設定.
     * <BR><BR>
     * 対象のバイナリ情報を設定します.
     * <BR>
     * @param no 設定開始位置となる項番を設定します.
     * @param bin 設定対象のバイナリ情報を設定します.
     * @param off 設定対象のオフセット値を設定します.
     * @param len 設定対象のバイナリ長を設定します.
     * @return int 設定されたバイナリ長が返されます.
     * @exception ArrayIndexOutOfBoundsException 不正インデックス例外.
     */
    public final int setBinary( int no,byte[] bin,int off,int len ) throws ArrayIndexOutOfBoundsException
    {
        int inputLen ;
        int ret ;
        
        if( bin == null || off < 0 || len <= 0 || no < 0 ){
            if( bin == null ){
                throw new ArrayIndexOutOfBoundsException( "設定バイナリデータは不正です" ) ;
            }
            else if( no < 0 ){
                throw new ArrayIndexOutOfBoundsException(
                    "指定位置(" + no + "は範囲外です"
                ) ;
            }
            else{
                throw new ArrayIndexOutOfBoundsException(
                    "設定条件(off:" + off + " len:" + len + ")は不正です"
                ) ;
            }
        }
        else if( ( inputLen = bin.length ) <= 0 ){
            return 0 ;
        }
        else if( off >= inputLen ){
            throw new ArrayIndexOutOfBoundsException(
                "設定条件(off:" + off +
                ")は設定バイナリ長(" + inputLen +
                ")を越しています"
            ) ;
        }
        
        try{
            
            synchronized( m_sync.get() ){
                
                inputLen = ( ( off + len ) > inputLen ) ? inputLen - off : len ;
                this.newSize( inputLen + no ) ;
                m_fp.write( bin,( long )no,off,inputLen ) ;
                m_usePoint = ( m_usePoint <= ( no + inputLen ) ) ? ( no + inputLen ) : m_usePoint ;
                
                ret = inputLen ;
                
            }
        }catch( ArrayIndexOutOfBoundsException ai ){
            throw ai ;
        }catch( Exception in ){
            ret = 0 ;
        }
        
        return ret ;
    }
    
    /**
     * バイナリ情報設定.
     * <BR><BR>
     * 対象のバイナリ情報を設定します.
     * <BR>
     * @param no 設定開始位置となる項番を設定します.
     * @param bin 設定対象のバイナリオブジェクトを設定します.
     * @return int 設定されたバイナリ長が返されます.
     * @exception ArrayIndexOutOfBoundsException 不正インデックス例外.
     */
    public final int setBinary( int no,BinResource bin ) throws ArrayIndexOutOfBoundsException
    {
        if( bin == null || bin.isUse() == false || no < 0 ){
            if( bin == null || bin.isUse() == false ){
                throw new ArrayIndexOutOfBoundsException( "設定バイナリデータは不正です" ) ;
            }
            throw new ArrayIndexOutOfBoundsException(
                "指定位置(" + no + "は範囲外です"
            ) ;
        }
        else if( bin.size() <= 0 ){
            return 0 ;
        }
        
        return this.setBinary( no,bin,0,bin.size() ) ;
    }
    
    /**
     * バイナリ情報設定.
     * <BR><BR>
     * 対象のバイナリ情報を設定します.
     * <BR>
     * @param no 設定開始位置となる項番を設定します.
     * @param bin 設定対象のバイナリオブジェクトを設定します.
     * @param off 設定対象のオフセット値を設定します.
     * @param len 設定対象のバイナリ長を設定します.
     * @return int 設定されたバイナリ長が返されます.
     * @exception ArrayIndexOutOfBoundsException 不正インデックス例外.
     */
    public final int setBinary( int no,BinResource bin,int off,int len ) throws ArrayIndexOutOfBoundsException
    {
        int i ;
        int lenI ;
        int inputLen ;
        int etc ;
        int pnt ;
        int bpnt ;
        int wlen ;
        int ret ;
        
        RandomIO fp = null ;
        byte[] buf = null ;
        
        if( bin == null || bin.isUse() == false || off < 0 || len <= 0 || no < 0 ){
            if( bin == null || bin.isUse() == false ){
                throw new ArrayIndexOutOfBoundsException( "設定バイナリデータは不正です" ) ;
            }
            else if( no < 0 ){
                throw new ArrayIndexOutOfBoundsException(
                    "指定位置(" + no + "は範囲外です"
                ) ;
            }
            else{
                throw new ArrayIndexOutOfBoundsException(
                    "設定条件(off:" + off + " len:" + len + ")は不正です"
                ) ;
            }
        }
        else if( ( inputLen = bin.size() ) <= 0 ){
            return 0 ;
        }
        else if( off >= inputLen ){
            throw new ArrayIndexOutOfBoundsException(
                "設定条件(off:" + off +
                ")は設定バイナリ長(" + inputLen +
                ")を越しています"
            ) ;
        }
        
        
        try{
            
            synchronized( m_sync.get() ){
                
                fp = m_fp ;
                
                inputLen = ( ( off + len ) > inputLen ) ? inputLen - off : len ;
                this.newSize( inputLen + no ) ;
                
                lenI = inputLen / ResourceDef.BUFFER_LENGTH ;
                etc = inputLen % ResourceDef.BUFFER_LENGTH ;
                
                buf = new byte[ ResourceDef.BUFFER_LENGTH ] ;
                for( i = 0,pnt = no,bpnt = off,ret = 0 ; i < lenI ; i ++ ){
                    
                    wlen = bin.getBinary( buf,bpnt ) ;
                    fp.write( buf,( long )pnt,0,wlen ) ;
                    
                    pnt += wlen ;
                    bpnt += wlen ;
                    ret += wlen ;
                    
                }
                
                if( etc != 0 ){
                    wlen = bin.getBinary( buf,bpnt,0,etc ) ;
                    fp.write( buf,( long )pnt,0,wlen ) ;
                    ret += wlen ;
                }
                
                m_usePoint = ( m_usePoint <= ( no + inputLen ) ) ? ( no + inputLen ) : m_usePoint ;
            }
        }catch( ArrayIndexOutOfBoundsException ai ){
            throw ai ;
        }catch( Exception in ){
            ret = 0 ;
        }finally{
            fp = null ;
            buf = null ;
        }
        
        return ret ;
    }
    
    /**
     * 情報取得.
     * <BR><BR>
     * 対象条件の情報を取得します.
     * <BR>
     * @param no 取得対象項番を設定します.
     * @return int 対象のバイナリ情報が返されます.
     * @exception ArrayIndexOutOfBoundsException 不正インデックス例外.
     */
    public final int get( int no ) throws ArrayIndexOutOfBoundsException
    {
        int ret ;
        byte[] tmp = null ;
        
        if( no < 0 ){
            throw new ArrayIndexOutOfBoundsException(
                "指定位置(" + no + "は範囲外です"
            ) ;
        }
        
        try{
            synchronized( m_sync.get() ){
                
                if( m_usePoint <= no ){
                    throw new ArrayIndexOutOfBoundsException(
                        "指定位置(" + no +
                        ")は有効データ長(" + m_usePoint +
                        ")を越しています"
                    ) ;
                }
                
                tmp = m_fp.read( ( long )no,1 ) ;
                ret = ( int )( tmp[ 0 ] & 0x000000ff ) ;
                
            }
        }catch( ArrayIndexOutOfBoundsException ai ){
            throw ai ;
        }catch( Exception in ){
            ret = -1 ;
        }finally{
            tmp = null ;
        }
        
        return ret ;
    }
    
    /**
     * バイナリ情報を取得.
     * <BR><BR>
     * 格納されているバイナリ情報を取得します.
     * <BR>
     * @return byte[] 設定されているバイナリ情報が返されます.
     */
    public final byte[] getBinary()
    {
        byte[] ret = null ;
        
        try{
            synchronized( m_sync.get() ){
                if( m_usePoint <= 0 ){
                    return null ;
                }
                ret = m_fp.read( 0L,m_usePoint ) ;
            }
            
        }catch( Exception in ){
            ret = null ;
        }
        
        return ret ;
    }
    
    /**
     * バイナリ情報を取得.
     * <BR><BR>
     * 対象のバイナリ情報を取得します.
     * <BR>
     * @param no 取得開始位置となる項番を設定します.
     * @return byte[] 取得されたバイナリ情報が返されます.
     * @exception ArrayIndexOutOfBoundsException 不正インデックス例外.
     */
    public final byte[] getBinary( int no ) throws ArrayIndexOutOfBoundsException
    {
        int len ;
        byte[] ret = null ;
        
        if( no < 0 ){
            throw new ArrayIndexOutOfBoundsException(
                "指定位置(" + no + "は範囲外です"
            ) ;
        }
        
        try{
            synchronized( m_sync.get() ){
                
                if( m_usePoint <= no ){
                    throw new ArrayIndexOutOfBoundsException(
                        "指定位置(" + no +
                        ")は有効データ長(" + m_usePoint +
                        ")を越しています"
                    ) ;
                }
                
                len = m_usePoint ;
                ret = m_fp.read( ( long )no,len-no ) ;
                
            }
        }catch( ArrayIndexOutOfBoundsException ai ){
            throw ai ;
        }catch( Exception in ){
            ret = null ;
        }
        
        return ret ;
    }
    
    /**
     * バイナリ情報を取得.
     * <BR><BR>
     * 対象のバイナリ情報を取得します.
     * <BR>
     * @param no 取得開始位置となる項番を設定します.
     * @param len 取得対象のバイナリ長を設定します.
     * @return byte[] 取得されたバイナリ情報が返されます.
     * @exception ArrayIndexOutOfBoundsException 不正インデックス例外.
     */
    public final byte[] getBinary( int no,int len ) throws ArrayIndexOutOfBoundsException
    {
        byte[] ret = null ;
        
        if( no < 0 ){
            throw new ArrayIndexOutOfBoundsException(
                "指定位置(" + no + "は範囲外です"
            ) ;
        }
        else if( len <= 0 ){
            return null ;
        }
        
        try{
            synchronized( m_sync.get() ){
                
                if( m_usePoint <= no ){
                    throw new ArrayIndexOutOfBoundsException(
                        "指定位置(" + no +
                        ")は有効データ長(" + m_usePoint +
                        ")を越しています"
                    ) ;
                }
                
                len = ( m_usePoint < no + len ) ? m_usePoint - no : len ;
                ret = m_fp.read( ( long )no,len ) ;
                
            }
        }catch( ArrayIndexOutOfBoundsException ai ){
            throw ai ;
        }catch( Exception in ){
            ret = null ;
        }
        
        return ret ;
    }
    
    /**
     * バイナリ情報を取得.
     * <BR><BR>
     * 格納されているバイナリ情報を取得します.
     * <BR>
     * @param out 取得対象のバイナリ情報が返されます.
     * @return int 取得されたバイナリ長が返されます.
     */
    public final int getBinary( byte[] out )
    {
        int inputLen ;
        
        if( out == null ){
            throw new ArrayIndexOutOfBoundsException( "設定バイナリデータは不正です" ) ;
        }
        else if( ( inputLen = out.length ) <= 0 ){
            return 0 ;
        }
        
        return this.getBinary( out,0,0,inputLen ) ;
    }
    
    /**
     * バイナリ情報を取得.
     * <BR><BR>
     * 対象のバイナリ情報を取得します.
     * <BR>
     * @param out 取得対象のバイナリ情報が返されます.
     * @param no 取得開始位置となる項番を設定します.
     * @return int 取得されたバイナリ長が返されます.
     * @exception ArrayIndexOutOfBoundsException 不正インデックス例外.
     */
    public final int getBinary( byte[] out,int no ) throws ArrayIndexOutOfBoundsException
    {
        int inputLen ;
        
        if( out == null ){
            throw new ArrayIndexOutOfBoundsException( "設定バイナリデータは不正です" ) ;
        }
        else if( ( inputLen = out.length ) <= 0 ){
            return 0 ;
        }
        
        return this.getBinary( out,no,0,inputLen ) ;
    }
    
    /**
     * バイナリ情報を取得.
     * <BR><BR>
     * 対象のバイナリ情報を取得します.
     * <BR>
     * @param out 取得対象のバイナリ情報が返されます.
     * @param no 取得開始位置となる項番を設定します.
     * @param off 取得対象のバイナリオフセット値を設定します.
     * @param len 取得対象のバイナリ長を設定します.
     * @return int 取得されたバイナリ長が返されます.
     * @exception ArrayIndexOutOfBoundsException 不正インデックス例外.
     */
    public final int getBinary( byte[] out,int no,int off,int len ) throws ArrayIndexOutOfBoundsException
    {
        int inputLen ;
        int ret ;
        
        if( out == null || off < 0 || len <= 0 || no < 0 ){
            if( out == null ){
                throw new ArrayIndexOutOfBoundsException( "設定バイナリデータは不正です" ) ;
            }
            else if( no < 0 ){
                throw new ArrayIndexOutOfBoundsException(
                    "指定位置(" + no + "は範囲外です"
                ) ;
            }
            else{
                throw new ArrayIndexOutOfBoundsException(
                    "設定条件(off:" + off + " len:" + len + ")は不正です"
                ) ;
            }
        }
        else if( ( inputLen = out.length ) <= 0 ){
            return 0 ;
        }
        
        try{
            
            synchronized( m_sync.get() ){
                
                inputLen = ( ( off + len ) > inputLen ) ? inputLen - off : len ;
                inputLen = ( m_usePoint < no + inputLen ) ? m_usePoint - no : inputLen ;
                ret = m_fp.read( out,( long )no,off,inputLen ) ;
                
            }
        }catch( ArrayIndexOutOfBoundsException ai ){
            throw ai ;
        }catch( Exception in ){
            ret = 0 ;
        }
        
        return ret ;
    }
    
    /**
     * バイナリ情報を取得.
     * <BR><BR>
     * 格納されているバイナリ情報を取得します.
     * <BR>
     * @param out 取得対象のバイナリ情報が返されます.
     * @return int 取得されたバイナリ長が返されます.
     */
    public final int getBinary( BinResource out )
    {
        int inputLen ;
        
        if( out == null || out.isUse() == false ){
            throw new ArrayIndexOutOfBoundsException( "設定バイナリデータは不正です" ) ;
        }
        else if( ( inputLen = out.getAllSize() ) <= 0 ){
            return 0 ;
        }
        
        return this.getBinary( out,0,0,inputLen ) ;
    }
    
    /**
     * バイナリ情報を取得.
     * <BR><BR>
     * 対象のバイナリ情報を取得します.
     * <BR>
     * @param out 取得対象のバイナリ情報が返されます.
     * @param no 取得開始位置となる項番を設定します.
     * @return int 取得されたバイナリ長が返されます.
     * @exception ArrayIndexOutOfBoundsException 不正インデックス例外.
     */
    public final int getBinary( BinResource out,int no ) throws ArrayIndexOutOfBoundsException
    {
        int inputLen ;
        
        if( out == null || out.isUse() == false ){
            throw new ArrayIndexOutOfBoundsException( "設定バイナリデータは不正です" ) ;
        }
        else if( ( inputLen = out.getAllSize() ) <= 0 ){
            return 0 ;
        }
        
        return this.getBinary( out,no,0,inputLen ) ;
    }
    
    /**
     * バイナリ情報を取得.
     * <BR><BR>
     * 対象のバイナリ情報を取得します.
     * <BR>
     * @param out 取得対象のバイナリ情報が返されます.
     * @param no 取得開始位置となる項番を設定します.
     * @param off 取得対象のバイナリオフセット値を設定します.
     * @param len 取得対象のバイナリ長を設定します.
     * @return int 取得されたバイナリ長が返されます.
     * @exception ArrayIndexOutOfBoundsException 不正インデックス例外.
     */
    public final int getBinary( BinResource out,int no,int off,int len ) throws ArrayIndexOutOfBoundsException
    {
        int inputLen ;
        int ret ;
        
        if( out == null || out.isUse() == false || off < 0 || len <= 0 || no < 0 ){
            if( out == null || out.isUse() == false ){
                throw new ArrayIndexOutOfBoundsException( "設定バイナリデータは不正です" ) ;
            }
            else if( no < 0 ){
                throw new ArrayIndexOutOfBoundsException(
                    "指定位置(" + no + "は範囲外です"
                ) ;
            }
            else{
                throw new ArrayIndexOutOfBoundsException(
                    "設定条件(off:" + off + " len:" + len + ")は不正です"
                ) ;
            }
        }
        else if( ( inputLen = out.getAllSize() ) <= 0 ){
            return 0 ;
        }
        
        try{
            
            synchronized( m_sync.get() ){
                
                //inputLen = ( ( off + len ) > inputLen ) ? inputLen - off : len ;
                //inputLen = ( m_usePoint < no + inputLen ) ? m_usePoint - no : inputLen ;
                inputLen = ( m_usePoint < no + len ) ? m_usePoint - no : len ;
                ret = out.setBinary( off,this,no,inputLen ) ;
                
            }
        }catch( ArrayIndexOutOfBoundsException ai ){
            throw ai ;
        }catch( Exception in ){
            ret = 0 ;
        }
        
        return ret ;
    }
    
    /**
     * 現在の有効バイナリ長を取得.
     * <BR><BR>
     * 現在の有効なバイナリ長を取得します.
     * <BR>
     * @return int 現在の有効なバイナリ長が返されます.
     */
    public final int size()
    {
        int ret ;
        
        try{
            synchronized( m_sync.get() ){
                ret = m_usePoint ;
            }
        }catch( Exception e ){
            ret = 0 ;
        }
        
        return ret ;
    }
    
    /**
     * 現在のバイナリ長を取得.
     * <BR><BR>
     * 現在のバイナリ長を取得します.
     * <BR>
     * @return int 現在のバイナリ長が返されます.
     */
    public final int getAllSize()
    {
        int ret ;
        
        try{
            synchronized( m_sync.get() ){
                ret = m_allLength ;
            }
        }catch( Exception e ){
            ret = 0 ;
        }
        
        return ret ;
    }
    
    /**
     * オブジェクトタイプを取得.
     * <BR><BR>
     * オブジェクトタイプを取得します.
     * <BR>
     * @return int オブジェクトタイプが返されます.<BR>
     *             [BinResource#BIN_RESOURCE_TYPE_MEMORY]が返された場合、
     *             [com.JRcServer.commons.resource.BinMemoryResource]オブジェクトです.<BR>
     *             [BinResource#BIN_RESOURCE_TYPE_FILE]が返された場合、
     *             [com.JRcServer.commons.resource.BinFileResource]オブジェクトです.<BR>
     *             [BinResource#BIN_RESOURCE_TYPE_CACHE]が返された場合、
     *             [com.JRcServer.commons.resource.BinCacheResource]オブジェクトです.
     */
    public final int getType()
    {
        return BinResource.BIN_RESOURCE_TYPE_FILE ;
    }
    
    /**
     * オブジェクト有効チェック.
     * <BR><BR>
     * オブジェクトが有効であるかチェックします.
     * <BR>
     * @return boolean チェック結果が返されます.<BR>
     *                 [true]が返された場合有効です.<BR>
     *                 [false]が返された場合無効です.
     */
    public final boolean isUse()
    {
        boolean ret ;
        
        try{
            synchronized( m_sync.get() ){
                ret = m_fp.isOpen() ;
            }
        }catch( Exception e ){
            ret = false ;
        }
        
        return ret ;
    }
    
    
    /**
     * バイナリ長を増やす.
     */
    private final void newSize( int size ) throws InputException
    {
        
        if( size <= 0 ){
            throw new InputException( "引数は不正です" ) ;
        }
        
        try{
            
            if( this.isUse() == true ){
                
                if( m_allLength < size ){
                    
                    size = ( size / BINARY_LENGTH ) + ( size % BINARY_LENGTH ) ;
                    size *= BINARY_LENGTH ;
                    
                    m_fp.setLength( size ) ;
                    
                    m_allLength = size ;
                    
                }
                
            }
            
        }catch( Exception e ){
            throw new InputException( e ) ;
        }
        
    }
    
    /**
     * ランダムファイル名を生成.
     */
    private static final String createRandomFile( String dir,RandomIO fp )
        throws InputException,FileAccessException
    {
        int i ;
        
        StringBuffer buf = null ;
        String randName = null ;
        String randFPath = null ;
        
        
        for( i = 0 ; i < ResourceDef.RETRY_RANDOM_NAME ; i ++ ){
            
            synchronized( Resource.SYNC ) {
                
                randName = UtilCom.randomString( ResourceDef.RANDOM_FILE_NAME,true,true,true ) ;
                
                buf = new StringBuffer() ;
                buf.append( dir ) ;
                buf.append( BaseDef.FILE_SEPARATOR ) ;
                buf.append( randName ) ;
                
                randFPath = buf.toString() ;
                buf = null ;
                
                if( IOCom.isFileExists( randFPath ) == false ){
                    break ;
                }
                
                randFPath = null ;
                
            }
            
        }
        
        if( randFPath == null ){
            throw new FileAccessException( "ランダムファイル名の生成に失敗しました" ) ;
        }
        
        fp.open( randFPath,true ) ;
        return randName ;
    }
    
}
